<?php
// Generate a labelled graph based on CSV financial data

// Image size and margin
$dim    = array('x' => 1600, 'y' => 1200);
$margin = 100;

// Read in the CSV
$x = array();
$y = array();

$fp = fopen('ftse.csv', 'r');

for ($i = 0; $row = fgetcsv($fp); $i++) {
    $x[$i] = strtotime($row[0]);
    $y[$i] = $row[4];
}

$min = min($y);
$max = max($y);

// Work out graph size
$size = array(
    'x' => $dim['x'] - $margin * 2,
    'y' => $dim['y'] - $margin * 2,
);

// Calculate the width of a step on each axis
$step = array(
    'x' => $size['x'] / (count($y) - 1),
    'y' => $size['y'] / ($max - $min),
);

// Calculate pixel positions for graph plot
// Note: Y-values are reversed; origin is at top-left
$pixels = array();

foreach ($y as $k => $v) {
    $pixels[$k] = array(
        'x' => (int)($k * $step['x']) + $margin,
        'y' => $size['y'] - (int)(($v - $min) * $step['y']) + $margin,
    );
}

// Set colours
$black = new ImagickPixel('black');
$white = new ImagickPixel('white');
$gray =  new ImagickPixel('#666666');
$light = new ImagickPixel('#cccccc');

// Make a blank image
$image = new Imagick();
$image->newImage($dim['x'], $dim['y'], $white);

// Draw the plot
$draw = new ImagickDraw();
$draw->setStrokeColor($black);
$draw->setFillColor($light);
$draw->setFontFamily('Tahoma');
$draw->setFontSize(48);
$draw->setGravity(Imagick::GRAVITY_NORTH);
$draw->annotation(0, 0, 'FTSE Index for 1H 2007');

// Draw X-axis labels, with rotated text
// Labels placed every 10 plot values
// Coordinate system rotates around top-left corner,
// so X=-Y and Y=X for plotting text position
$draw->setGravity(Imagick::GRAVITY_NORTHEAST);
$draw->setFillColor($gray);
$draw->setFontSize(24);
$draw->rotate(-90);

for($i = 0; $i < count($pixels); $i += 10) {
    $draw->annotation(
       - ($dim['y'] - $margin + 10),
          $dim['x'] - $pixels[$i]['x'],
          date('j M', $x[$i]));
}

// Bring coordinate system back to non-rotation
$draw->rotate(90);

// Draw axis lines
$draw->setStrokeColor($gray);
$draw->setFillColor($white);
$draw->line(
    $margin, $margin,
    $margin, $margin + $size['y']
);
$draw->line(
    $margin, $margin + $size['y'],
    $margin + $size['x'], $margin + $size['y']
);

// Draw the plot
$draw->setStrokeColor($black);
$draw->polyline($pixels);

// Draw Y-axis labels every 100
// Also draws Y-value guide lines
$split = round($min, -2);

if ($split < $min) {
    $split += 100;
}

do {
    $draw->setStrokeColor($light);
    $draw->setFillColor($light);
    $draw->setStrokeWidth(1);
    $draw->line(
        $margin,
        $margin + $size['y'] - ($split - $min) * $step['y'],
        $margin + $size['x'],
        $margin + $size['y'] - ($split - $min) * $step['y']
    );

    $draw->setStrokeColor($black);
    $draw->setFillColor($gray);
    $draw->annotation(
        $margin + $size['x'] + 5,
        $margin + $size['y'] - ($split - $min) * $step['y'],
        $split
    );

    $split += 100;
} while ($split < $max);

// Apply draw commands to image, and output
$image->drawImage($draw);
$image->setImageFormat('png');
$image->writeImage('ftse-graph-labelled.png');

?>
